@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2022 - pancake */
+/* radare - LGPL - Copyright 2009-2023 - pancake */
 
 #ifndef R2_DEBUG_H
 #define R2_DEBUG_H
@@ -25,8 +25,10 @@ R_LIB_VERSION_HEADER(r_debug);
 
 /* hack to fix compilation of debugger on BSD systems */
 /* This needs some testing (netbsd, freebsd, openbsd, kfreebsd) */
-#if __BSD__
+#if R2__BSD__ || defined(__serenity__)
+#if R2__BSD__
 #include <machine/reg.h>
+#endif
 
 /* hakish hack to hack the openbsd/sparc64 hack */
 #undef reg
@@ -172,6 +174,8 @@ typedef struct r_debug_snap_t {
 	int perm;
 	int user;
 	bool shared;
+	ut32 crc;
+	char *comment;
 } RDebugSnap;
 
 typedef struct {
@@ -224,14 +228,14 @@ typedef struct r_debug_trace_t {
 	RList *traces;
 	int count;
 	int enabled;
-	//int changed;
 	int tag;
 	int dup;
 	char *addresses;
-	// TODO: add range here
-	HtPP *ht;
+	HtPP *ht; // use rbtree like the iocache?
 } RDebugTrace;
 
+// R2_590 rename to traceitem for consistency?
+#define r_debug_tracepoint_free(x) free((x))
 typedef struct r_debug_tracepoint_t {
 	ut64 addr;
 	ut64 tags; // XXX
@@ -240,11 +244,16 @@ typedef struct r_debug_tracepoint_t {
 	int count;
 	int times;
 	ut64 stamp;
+#if 0
+	// registers accessed
+	// memory access
+	ut64 refaddr;
+	int direction
+#endif
 } RDebugTracepoint;
 
 typedef struct r_debug_t {
-	// R2_570
-	// use RArchConfig here
+	// R2_590 use RArchConfig instead
 	char *arch;
 	int bits; /// XXX: MUST SET ///
 	int hitinfo;
@@ -254,9 +263,9 @@ typedef struct r_debug_t {
 	int tid; /* selected thread id */
 	int forked_pid; /* last pid created by fork */
 	int n_threads;
-	RList *threads; /* NOTE: list contents are platform-specific */
+	RList *threads; // NOTE: list contents are platform-specific
 
-	char *malloc;     /*choose malloc parser: 0 = glibc, 1 = jemalloc*/
+	char *malloc; // choose malloc parser: 0 = glibc, 1 = jemalloc
 
 	/* dbg.* config options (see e?dbg)
 	 * NOTE: some settings are checked inline instead of tracked here.
@@ -311,6 +320,7 @@ typedef struct r_debug_t {
 	RAnal *anal;
 	RList *maps; // <RDebugMap>
 	RList *maps_user; // <RDebugMap>
+	RList *snaps; // <RDebugSnap> -- user defined snapshots
 
 	bool trace_continue;
 	RAnalOp *cur_op;
@@ -367,6 +377,9 @@ typedef struct r_debug_plugin_t {
 	const char *license;
 	const char *author;
 	const char *version;
+#if R2_590
+	const char *description;
+#endif
 	//const char **archs; // MUST BE DEPRECATED!!!!
 	ut32 bits;
 	const char *arch;
@@ -381,7 +394,7 @@ typedef struct r_debug_plugin_t {
 	RList *(*threads)(RDebug *dbg, int pid);
 	RList *(*pids)(RDebug *dbg, int pid);
 	RList *(*tids)(RDebug *dbg, int pid);
-	RFList (*backtrace)(RDebug *dbg, int count);
+	RList (*backtrace)(RDebug *dbg, int count);
 	/* flow */
 	int (*stop)(RDebug *dbg);
 	bool (*step)(RDebug *dbg);
@@ -394,9 +407,8 @@ typedef struct r_debug_plugin_t {
 	bool (*contsc)(RDebug *dbg, int pid, int sc);
 	RList* (*frames)(RDebug *dbg, ut64 at);
 	RBreakpointCallback breakpoint;
-// XXX: specify, pid, tid, or RDebug ?
-	int (*reg_read)(RDebug *dbg, int type, ut8 *buf, int size);
-	int (*reg_write)(RDebug *dbg, int type, const ut8 *buf, int size); //XXX struct r_regset_t regs);
+	bool (*reg_read)(RDebug *dbg, int type, ut8 *buf, int size);
+	bool (*reg_write)(RDebug *dbg, int type, const ut8 *buf, int size);
 	char* (*reg_profile)(RDebug *dbg);
 	int (*set_reg_profile)(const char *str);
 	/* memory */
@@ -436,13 +448,13 @@ R_API RDebug *r_debug_new(int hard);
 R_API void r_debug_free(RDebug *dbg);
 
 R_API bool r_debug_attach(RDebug *dbg, int pid);
-R_API int r_debug_detach(RDebug *dbg, int pid);
-R_API int r_debug_startv(RDebug *dbg, int argc, char **argv);
-R_API int r_debug_start(RDebug *dbg, const char *cmd);
+R_API bool r_debug_detach(RDebug *dbg, int pid);
+R_API bool r_debug_startv(RDebug *dbg, int argc, char **argv);
+R_API bool r_debug_start(RDebug *dbg, const char *cmd);
 
 /* reason we stopped */
 R_API RDebugReasonType r_debug_stop_reason(RDebug *dbg);
-R_API const char *r_debug_reason_to_string(int type);
+R_API const char *r_debug_reason_tostring(int type);
 
 /* wait for another event */
 R_API RDebugReasonType r_debug_wait(RDebug *dbg, RBreakpointItem **bp);
@@ -459,7 +471,7 @@ R_API int r_debug_continue_syscall(RDebug *dbg, int sc);
 R_API int r_debug_continue_syscalls(RDebug *dbg, int *sc, int n_sc);
 R_API int r_debug_continue(RDebug *dbg);
 R_API int r_debug_continue_kill(RDebug *dbg, int signal);
-#if __WINDOWS__
+#if R2__WINDOWS__
 R_API int r_debug_continue_pass_exception(RDebug *dbg);
 #endif
 
@@ -484,6 +496,7 @@ R_API ut64 r_debug_get_baddr(RDebug *dbg, const char *file);
 
 /* send signals */
 R_API void r_debug_signal_init(RDebug *dbg);
+R_API void r_debug_signal_fini(RDebug *dbg);
 R_API int r_debug_signal_send(RDebug *dbg, int num);
 R_API int r_debug_signal_what(RDebug *dbg, int num);
 R_API int r_debug_signal_resolve(RDebug *dbg, const char *signame);
@@ -607,6 +620,14 @@ R_API ut8 *r_debug_snap_get_hash(RDebugSnap *snap);
 R_API bool r_debug_snap_is_equal(RDebugSnap *a, RDebugSnap *b);
 R_API void r_debug_snap_free(RDebugSnap *snap);
 
+/* snap */
+R_API int r_debug_snap_delete(RDebug *dbg, int idx);
+R_API void r_debug_snap_list(RDebug *dbg, int idx, int mode);
+R_API int r_debug_snap_diff(RDebug *dbg, int idx);
+R_API int r_debug_snap(RDebug *dbg, ut64 addr);
+R_API int r_debug_snap_comment(RDebug *dbg, int idx, const char *msg);
+R_API int r_debug_snap_all(RDebug *dbg, int perms);
+
 R_API int r_debug_step_back(RDebug *dbg, int steps);
 R_API bool r_debug_goto_cnum(RDebug *dbg, ut32 cnum);
 R_API int r_debug_step_cnum(RDebug *dbg, int steps);
@@ -635,6 +656,7 @@ extern RDebugPlugin r_debug_plugin_windbg;
 extern RDebugPlugin r_debug_plugin_bochs;
 extern RDebugPlugin r_debug_plugin_qnx;
 extern RDebugPlugin r_debug_plugin_null;
+extern RDebugPlugin r_debug_plugin_rv32ima;
 #endif
 
 #ifdef __cplusplus
