@@ -12,7 +12,7 @@ typedef struct r_mmap_t {
 	int fd;
 	int rw;
 	char *filename;
-#if __WINDOWS__
+#if R2__WINDOWS__
 	HANDLE fh;
 	HANDLE fm;
 #endif
@@ -38,11 +38,11 @@ R_API void* r_mem_pool_alloc(RMemoryPool *pool);
 R_API void *r_mem_dup(const void *s, int l);
 R_API void *r_mem_alloc(int sz);
 R_API void r_mem_free(void *);
-R_API void r_mem_memzero(void *, size_t);
+R_API void r_mem_zero(void *, size_t);
 R_API void r_mem_reverse(ut8 *b, int l);
 R_API bool r_mem_protect(void *ptr, int size, const char *prot);
-R_API int r_mem_set_num(ut8 *dest, int dest_size, ut64 num);
-R_API int r_mem_eq(ut8 *a, ut8 *b, int len);
+R_API bool r_mem_set_num(ut8 *dest, int dest_size, ut64 num);
+R_API bool r_mem_eq(ut8 *a, ut8 *b, int len);
 R_API void r_mem_copybits(ut8 *dst, const ut8 *src, int bits);
 R_API void r_mem_copybits_delta(ut8 *dst, int doff, const ut8 *src, int soff, int bits);
 R_API void r_mem_copyloop(ut8 *dest, const ut8 *orig, int dsize, int osize);
@@ -57,6 +57,17 @@ R_API bool r_mem_is_printable(const ut8 *a, int la);
 R_API bool r_mem_is_zero(const ut8 *b, int l);
 R_API void *r_mem_mmap_resize(RMmap *m, ut64 newsize);
 
+// 27bit middle endian parser
+typedef ut32 ut27;
+static inline ut27 r_read_me27(const ut8 *buf, int boff) {
+	ut27 ret = 0;
+	r_mem_copybits_delta ((ut8 *)&ret, 18, buf, boff, 9);
+	r_mem_copybits_delta ((ut8 *)&ret, 9, buf, boff + 9, 9);
+	r_mem_copybits_delta ((ut8 *)&ret, 0, buf, boff + 18, 9);
+	return ret;
+}
+
+
 #ifdef __cplusplus
 }
 #endif
