@@ -3,7 +3,10 @@
 
 #include <r_types.h>
 
-#if __WINDOWS__
+#ifdef __cplusplus
+extern "C" {
+#endif
+#if R2__WINDOWS__
 
 typedef enum r_w32dw_request {
 	W32_NONE,
@@ -52,4 +55,8 @@ R_API void r_w32dw_free(RW32Dw *inst);
 
 #endif
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
