@@ -11,12 +11,14 @@ extern "C" {
 #endif
 
 #include "types.h"
-#include "sdbht.h"
+#include "ht.h"
 #include "ls.h"
 #include "dict.h"
 #include "cdb.h"
 #include "cdb_make.h"
-#include "sdb_version.h"
+#include "version.h"
+#include "rangstr.h"
+#include "heap.h"
 
 /* Key value sizes */
 #define SDB_MIN_VALUE 1
@@ -162,6 +164,7 @@ SDB_API SdbList *sdb_foreach_list(Sdb* s, bool sorted);
 SDB_API SdbList *sdb_foreach_list_filter(Sdb* s, SdbForeachCallback filter, bool sorted);
 SDB_API SdbList *sdb_foreach_match(Sdb* s, const char *expr, bool sorted);
 
+SDB_API int sdb_main(int argc, const char **argv);
 SDB_API bool sdb_query(Sdb* s, const char *cmd);
 SDB_API int sdb_queryf(Sdb* s, const char *fmt, ...);
 SDB_API int sdb_query_lines(Sdb *s, const char *cmd);
@@ -171,6 +174,7 @@ SDB_API int sdb_query_file(Sdb *s, const char* file);
 SDB_API bool sdb_exists(Sdb*, const char *key);
 SDB_API bool sdb_remove(Sdb*, const char *key, ut32 cas);
 SDB_API int sdb_unset(Sdb*, const char *key, ut32 cas);
+SDB_API int sdb_nunset(Sdb*, ut64 nkey, ut32 cas);
 SDB_API int sdb_unset_like(Sdb *s, const char *k);
 SDB_API char** sdb_like(Sdb *s, const char *k, const char *v, SdbForeachCallback cb);
 
@@ -193,6 +197,7 @@ SDB_API bool sdb_diff(Sdb *a, Sdb *b, SdbDiffCallback cb, void *cb_user);
 
 // Gets a pointer to the value associated with `key`.
 SDB_API char *sdb_get(Sdb*, const char *key, ut32 *cas);
+SDB_API char *sdb_nget(Sdb*, ut64 nkey, ut32 *cas);
 
 // Gets a pointer to the value associated with `key` and returns in `vlen` the
 // length of the value string.
@@ -205,10 +210,14 @@ SDB_API const char *sdb_const_get(Sdb*, const char *key, ut32 *cas);
 // `vlen` the length of the value string.
 SDB_API const char *sdb_const_get_len(Sdb* s, const char *key, int *vlen, ut32 *cas);
 SDB_API int sdb_set(Sdb*, const char *key, const char *data, ut32 cas);
+SDB_API int sdb_nset(Sdb*, ut64 nkey, const char *data, ut32 cas);
+SDB_API ut64 sdb_num_nget(Sdb *s, ut64 nkey, ut32 *cas);
+SDB_API int sdb_num_nset(Sdb* s, ut64 nkey, ut64 nval, ut32 cas);
 SDB_API int sdb_set_owned(Sdb* s, const char *key, char *val, ut32 cas);
 SDB_API int sdb_concat(Sdb *s, const char *key, const char *value, ut32 cas);
 SDB_API int sdb_uncat(Sdb *s, const char *key, const char *value, ut32 cas);
 SDB_API int sdb_add(Sdb* s, const char *key, const char *val, ut32 cas);
+SDB_API int sdb_nadd(Sdb* s, ut64 nkey, const char *val, ut32 cas);
 SDB_API bool sdb_sync(Sdb*);
 SDB_API void sdbkv_free(SdbKv *kv);
 
@@ -254,7 +263,8 @@ SDB_API bool sdb_journal_clear(Sdb *s);
 SDB_API bool sdb_journal_unlink(Sdb *s);
 
 /* numeric */
-SDB_API char *sdb_itoa(ut64 n, char *s, int base);
+SDB_API char *sdb_itoa(ut64 n, int base, char *s, int slen);
+SDB_API char *sdb_itoas(ut64 n, int base);
 SDB_API ut64  sdb_atoi(const char *s);
 
 /* locking */
@@ -274,7 +284,7 @@ SDB_API ut32 sdb_hash_len(const char *key, ut32 *len);
 SDB_API ut8 sdb_hash_byte(const char *s);
 
 /* json api */
-// SDB_API int sdb_js0n(const unsigned char *js, RangstrType len, RangstrType *out);
+SDB_API int sdb_js0n(const unsigned char *js, RangstrType len, RangstrType *out);
 SDB_API bool sdb_isjson(const char *k);
 SDB_API char *sdb_json_get_str (const char *json, const char *path);
 SDB_API bool sdb_json_get_bool(const char *json, const char *path);
